{% extends "admin/base_site.html" %}
{% load i18n l10n admin_static %}
{% block extrastyle %}{{ block.super }}<link rel="stylesheet" type="text/css" href="{% static "admin/css/forms.css" %}">{% endblock %}

{% block breadcrumbs %}
<div class="breadcrumbs">
     <a href="../../">{% trans "Home" %}</a> &rsaquo;
     {% trans "OCR-innhenting" %}
</div>
{% endblock %}

{% block content %}
    <form action="" method="post">{% csrf_token %}
    <h1>Hent inn OCR-fil (registrer girobetalingar)</h1>

    <fieldset class="module aligned">
    <div class=form-row>
      <div>
        {% if errorlist %}
          <ul class=errorlist>
            {% for error in errorlist %}<li>{{ error }}{% endfor %}
          </ul>
        {% endif %}
        <label for=ocr_data class=required>OCR-data:</label>
        <textarea name=ocr_data class=vLargeTextField rows=25></textarea>
        <p class=help>Lim inn ei full OCR-køyring.
      </div>
    </div>
    </fieldset>

    <input type=hidden name=action value=import_ocr>
    <input type=hidden name=post value=yes>
    <div class=submit-row>
      <input type=submit value="Les inn OCR-fil" class=default>
    </div>
    </form>
{% endblock %}
