<div id="links">
  <a name="links"></a>
  <p>
    Ser eposten rar ut?
    <a href="http://{{ domain }}{% url newsletter_newsletter_contact slug=newsletter.slug,uidb36=uidb36,token=token %}">les han på nett</a>.
  </p>
</div>
