<!DOCTYPE html>
<html>
  <head>
    <title>Norsk Målungdom medlemsregister</title>
    <meta charset=utf-8>
<!--    <link rel="shortcut icon" href="/static/nynorsk-16.png"> -->
    {% block extrahead %}{% endblock %}

    <style type="text/css" media="all">
    body { font: normal 0.8em sans-serif;}
    #kontainer { position: relative; }
    thead { background-color: #333; color: #eee; }
    th, td { padding: 4px; }
    tr:nth-child(even) { background-color: #eee; }
    table { width: 100%; }

    .ikkje-teljande { color: #900; }
    .ikkje-teljande:after { content: ' ×'; }

    #site-title { margin: 0; padding: 0; }
    #site-title a { color: #544; font-size: 15pt; text-decoration: none; }
    nav ul { padding:0; margin: 0 0 0 200px; position: absolute; top: 10px; }
    nav li { display: inline-block; margin:0; width: 150px; text-align: center; }
    nav a { text-decoration: none; }

    footer { color: #999; }
    footer a { color: #999; }

.login h1 { text-align: center; }
.login-form { width: 300px; margin: 0 auto; box-shadow: 0 10px 20px -10px #ccc; }
.login-form ul { margin:0;padding:0; }
.login-form li { display: block; padding: 1em; margin: 0; }
.login-form label { display: inline-block; width: 100px; }
    </style>
    <style type="text/css" media="print">
#head { display: none; }
nav { display: none; }
#ekstra { display: none; }
#footer { display: none; }
    </style>

    <script type="text/javascript" src="/static/js/sorttable.js"></script>
    <script type="text/javascript">
    </script>

  </head>
<body class="{% block bodyclass %}standard{% endblock %}">

<div id="kontainer">

<div id="hovud">
  <div id=site-title><a href="/">Medlemssys</a></div>
</div>

{% block header %}
<nav>
  <ul>
    <li><a href="/">Framsida</a>
    <li><a href="http://malungdom.no/">Norsk Målungdom</a>
    <li><a href="http://www.nm.no">Noregs Mållag</a>
    <li><a href="/accounts/logout">Logg ut</a>
  </ul>
</nav>
{% endblock header %}

<div id="content">
	{% block content %}Ingenting her{% endblock %}
</div>

<aside id="venstrefelt">
</aside>

<footer>
	<p><small>System av <a href="http://velmont.no">Odin Hørthe Omdal</a></small></p>
</footer>

</div>

<!-- Piwik
<script type="text/javascript">
var pkBaseURL = (("https:" == document.location.protocol) ? "https://stats.bonarjee.in/" : "http://stats.bonarjee.in/");
document.write(unescape("%3Cscript src='" + pkBaseURL + "piwik.js' type='text/javascript'%3E%3C/script%3E"));
</script><script type="text/javascript">
try {
var piwikTracker = Piwik.getTracker(pkBaseURL + "piwik.php", 12);
piwikTracker.trackPageView();
piwikTracker.enableLinkTracking();
} catch( err ) {}
</script><noscript><p><img src="http://stats.bonarjee.in/piwik.php?idsite=12" style="border:0" alt=""/></p></noscript>
<!- End Piwik Tag -->
</body>
</html>
